@@ -1,7 +1,7 @@
 /*
     __ _____ _____ _____
  __|  |   __|     |   | |  JSON for Modern C++ (test suite)
-|  |  |__   |  |  | | | |  version 3.10.0
+|  |  |__   |  |  | | | |  version 3.10.4
 |_____|_____|_____|_|___|  https://github.com/nlohmann/json
 
 Licensed under the MIT License <http://opensource.org/licenses/MIT>.
@@ -184,4 +184,71 @@ TEST_CASE("Better diagnostics")
         j["second"] = value;
         j2["something"] = j;
     }
+
+    SECTION("Regression test for issue #3007 - Parent pointers properly set when using update()")
+    {
+        // void update(const_reference j)
+        {
+            json j = json::object();
+
+            {
+                json j2 = json::object();
+                j2["one"] = 1;
+
+                j.update(j2);
+            }
+
+            // Must call operator[] on const element, otherwise m_parent gets updated.
+            auto const& constJ = j;
+            CHECK_THROWS_WITH_AS(constJ["one"].at(0), "[json.exception.type_error.304] (/one) cannot use at() with number", json::type_error);
+        }
+
+        // void update(const_iterator first, const_iterator last)
+        {
+            json j = json::object();
+
+            {
+                json j2 = json::object();
+                j2["one"] = 1;
+
+                j.update(j2.begin(), j2.end());
+            }
+
+            // Must call operator[] on const element, otherwise m_parent gets updated.
+            auto const& constJ = j;
+            CHECK_THROWS_WITH_AS(constJ["one"].at(0), "[json.exception.type_error.304] (/one) cannot use at() with number", json::type_error);
+        }
+
+        // Code from #3007 triggering unwanted assertion without fix to update().
+        {
+            json root = json::array();
+            json lower = json::object();
+
+            {
+                json lowest = json::object();
+                lowest["one"] = 1;
+
+                lower.update(lowest);
+            }
+
+            root.push_back(lower);
+        }
+    }
+
+    SECTION("Regression test for https://github.com/nlohmann/json/issues/3032")
+    {
+        // reference operator[](size_type idx)
+        {
+            json j_arr = json::array();
+            j_arr[0] = 0;
+            j_arr[1] = 1;
+            j_arr[2] = 2;
+            j_arr[3] = 3;
+            j_arr[4] = 4;
+            j_arr[5] = 5;
+            j_arr[6] = 6;
+            j_arr[7] = 7;
+            json j_arr_copy = j_arr;
+        }
+    }
 }
